from django.apps import AppConfig


class WagtailSPAIntegrationAppConfig(AppConfig):
    name = "wagtail_spa_integration"
    verbose_name = "Wagtail SPA integration"
    default_auto_field = "django.db.models.AutoField"
